import { TableBody, TableCell, TableRow, Tooltip } from "@mui/material";
import moment from "moment";
import { memo } from "react";
import DeleteButton from "../../../UI/DeleteButton/DeleteButton";
import { getComparator, stableSort } from "./helpers";
import { Edit } from "@mui/icons-material";
import { useSelector } from "react-redux";

function UsersTasksTableBody({ rows, page, rowsPerPage, order, orderBy, deleteTaskHandler }) {
  const user = useSelector((state) => state.users.user);
  return (<>
    <TableBody>


      {stableSort(rows, getComparator(order, orderBy))
        .slice(page * rowsPerPage, page * rowsPerPage + rowsPerPage)
        .map((row, index) => {
          return (
            <TableRow
              hover
              key={row.id}
            >
              <TableCell sx={{ width: '4%' }}>{row.priority ? row.priority : null}</TableCell>
              <TableCell sx={{ width: '10%' }}>{moment(row.createdAt).format('DD.MM.YYYY')}</TableCell>
              <Tooltip title={row.description}>
                <TableCell sx={{ width: '25%', overflow: 'hidden', textOverflow: 'ellipsis' }}>
                  {row.title}
                </TableCell>
              </Tooltip>
              <TableCell sx={{ width: '10%' }}>{row.projectTitle}</TableCell>
              <TableCell sx={{ width: '8%' }}>{row.executorName}</TableCell>
              <TableCell sx={{ width: '8%' }}>{row.authorName}</TableCell>
              <TableCell sx={{ width: '15%' }}>{row.dateTimeTasks.length ? row.dateTimeTasks[0].dateTimeStart : null}</TableCell>
              <TableCell sx={{ width: '10%' }}>{row.dateTimeDeadLine ? moment(row.dateTimeDeadLine).utcOffset('+000').format('DD.MM.YYYY HH:MM') : null}</TableCell>
              <TableCell sx={{ width: '7%' }}>{row.accomplish}</TableCell>
              <TableCell size='medium' sx={{ width: '5%', position: 'relative' }}>
                {user.id === row.author.id ?
                  <>
                    <Edit sx={{ position: 'absolute', left: -10 }} />
                    <DeleteButton onClick={() => { deleteTaskHandler(row.id) }} />
                  </>
                  :
                  <Edit />
                }
              </TableCell>
            </TableRow>
          );
        })}
    </TableBody >
  </>);
}

export default memo(UsersTasksTableBody)